<?php
/**
 * Woo Integration Module
 * This class is responsible for Woo Integration module.
 * This is used to define internationalization, admin-specific hooks, and public-facing site hooks for Woo Integration module.
 *
 * @link       http://edwiser.org
 * @since      3.0.0
 * @package    Edwiser Bridge Pro
 */

namespace app\wisdmlabs\edwiserBridgePro\includes\wooInt;

use \app\wisdmlabs\edwiserBridge\EdwiserBridge;

/**
 * Bridge WooCommerce Course Class
 */
class Bridge_Woocommerce_Course {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Edwiser Bridge Instance
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      object    $edwiser_bridge    Edwiser Bridge Instance.
	 */
	private $edwiser_bridge;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name This plugin name.
	 * @param string $version     This plugin version.
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version     = $version;
		require_once WP_PLUGIN_DIR . '/edwiser-bridge/includes/class-eb.php';
		$this->edwiser_bridge = new EdwiserBridge();
	}

	/**
	 * This checks if Product for course on Moodle already created or not
	 *
	 * @param integer $course_id_on_moodle      This is Moodle course ID.
	 *
	 * @return integer/boolean $product_id  returns product ID if found, otherwise false
	 * @since   1.0.0
	 * @access  public
	 */
	public function is_product_presynced( $course_id_on_moodle ) {
		global $wpdb;

		$this->edwiser_bridge->logger()->add( 'product', 'Checking if a product is presynced, Moodle ID of course: ' . $course_id_on_moodle ); // add product log.

		$query = 'SELECT `product_id` FROM ' . $wpdb->prefix . "eb_moodle_course_products WHERE `moodle_course_id` = '" . $course_id_on_moodle . "'";

		$product_id = $wpdb->get_var( $query ); // @codingStandardsIgnoreLine

		$this->edwiser_bridge->logger()->add( 'product', 'Product Found? :' . ( ( $product_id ) ? 'Yes, the ID is: ' . $product_id : 'NO' ) ); // add product log.

		return $product_id ? $product_id : false;
	}

	/**
	 * This function creates products for already synchronized courses.
	 *
	 * @param array $sync_options synchronization option selected by User.
	 *
	 * @since   1.0.2
	 * @access  public
	 */
	public function bridge_woo_sync_create_product( $sync_options ) {
		global $wpdb;

		$query = 'SELECT `ID`, `post_title`, `post_content`
		FROM ' . $wpdb->prefix . "posts WHERE `post_type` = 'eb_course'
		AND `ID` NOT IN ( 
		
			SELECT DISTINCT `moodle_post_id` FROM `" . $wpdb->prefix . "eb_moodle_course_products` 
		 )
		AND `post_status` IN ( 'publish','draft' )";

		$result = $wpdb->get_results( $query ); // @codingStandardsIgnoreLine

		$response = array();

		if ( ! empty( $result ) ) {
			$post_status = ( isset( $sync_options['bridge_woo_synchronize_product_publish'] ) && 1 == $sync_options['bridge_woo_synchronize_product_publish'] ) ? 'publish' : 'private'; // @codingStandardsIgnoreLine

			foreach ( $result as $single_result ) {
				$course_name    = $this->wdm_isset( $single_result->post_title );
				$course_content = $this->wdm_isset( $single_result->post_content );
				$course_id      = $this->wdm_isset( $single_result->ID );

				if ( ! empty( $course_id ) ) {
					$this->edwiser_bridge->logger()->add( 'product', 'Creating Product for course id: ' . $course_id ); // Add product created log.

					$course_args = array(
						'post_title'   => $course_name,
						'post_content' => $course_content,
						'post_status'  => $post_status,
						'post_type'    => 'product',
					);

					$wp_product_id = wp_insert_post( $course_args ); // create a Product on WooCommerce
					// Add Product Meta.

					$moodle_course_id = get_post_meta( $course_id, 'moodle_course_id', true );

					$product_args = array(
						'moodle_course_id'      => $moodle_course_id,
						'moodle_post_course_id' => array( $course_id ),
					);

					update_post_meta( $wp_product_id, 'product_options', $product_args );

					// Make Product Virtual & Downloadable.

					add_post_meta( $wp_product_id, '_downloadable', 'yes' );
					add_post_meta( $wp_product_id, '_virtual', 'yes' );

					update_post_meta( $wp_product_id, 'is_product_a_moodle_course', 'yes' );

					// Update Course Meta.

					/*
					 * Change course status & Course closed url
					 */

					$eb_course_options = get_post_meta( $course_id, 'eb_course_options', true );

					$course_status_option = array(
						'course_price_type' => 'closed',
						'course_closed_url' => get_permalink( $wp_product_id ),
					);

					if ( ! empty( $eb_course_options ) ) {
						$eb_course_options = array_merge( $eb_course_options, $course_status_option );
					} else {
						$eb_course_options = $course_status_option;
					}

					update_post_meta( $course_id, 'eb_course_options', $eb_course_options );

					// Make Entry in Product Course Log.

					$woo_moo_course_tbl = $wpdb->prefix . 'eb_moodle_course_products';

					$data = array(
						'product_id'       => $wp_product_id,
						'moodle_post_id'   => $course_id,
						'moodle_course_id' => $moodle_course_id,
					);

					$wpdb->insert( $woo_moo_course_tbl, $data ); // @codingStandardsIgnoreLine

					// Add Product Log.
					$this->edwiser_bridge->logger()->add( 'product', 'Product created, ID is: ' . $wp_product_id ); // Add product created log.

					// Assign Product categories to Products.

					$category_args     = array(
						'orderby' => 'name',
						'order'   => 'ASC',
						'fields'  => 'slugs',
					);
					$course_categories = wp_get_object_terms( $course_id, 'eb_course_cat', $category_args );

					$this->set_obj_terms( $course_categories, $wp_product_id );
					do_action( 'bridge_woo_course_product_created', $wp_product_id, $course_id, $sync_options );
				}
			}

			$response['respone_message'] = '<div class="alert alert-success">' . __( 'Product( s ) created.', 'edwiser-bridge-pro' ) . '</div>';
		} else {
			$response['respone_message'] = '<div class="alert alert-warning"><span class="dashicons dashicons-warning" style="padding: 2px 6px 2px 0px;font-size: 22px;margin-left: -2px;"></span>' . __( 'Courses for synchronization not found. All products may be already created.', 'edwiser-bridge-pro' ) . '</div>';
		}

		return $response;
	}

	/**
	 * This function used to check isset(), to reduce NPath complexity.
	 *
	 * @param var $var  Variable to check.
	 * @return boolean true/false
	 * @since   1.0.2
	 */
	public function wdm_isset( $var ) {
		if ( isset( $var ) ) {
			return $var;
		} else {
			return '';
		}
	}

	/**
	 * This function sets object terms for product.
	 *
	 * @param array $course_categories   Course categories.
	 * @param int   $wp_product_id         Product ID.
	 *
	 * @return void
	 * @since   1.0.2
	 */
	public function set_obj_terms( $course_categories, $wp_product_id ) {
		if ( ! empty( $course_categories ) && ! is_wp_error( $course_categories ) ) {
			$product_term_id = array();

			foreach ( $course_categories as $single_course_cat ) {
				// Find corresponding Product category ID.

				$product_details = get_term_by( 'slug', $single_course_cat, 'product_cat', ARRAY_A );

				if ( isset( $product_details['term_id'] ) ) {
					$product_term_id[] = $product_details['term_id'];
				}
			}

			wp_set_object_terms( $wp_product_id, $product_term_id, 'product_cat', false );
			$this->edwiser_bridge->logger()->add( 'product', 'Categories assigned ' . implode( ',', $product_term_id ) ); // Add product created log.
		}
	}

	/**
	 * This function updates all existing WooCommerce product associated with Moodle Course.
	 *
	 * @param array $sync_options       synchronization option selected by User.
	 *
	 * @since   1.0.2
	 * @access  public
	 */
	public function bridge_woo_sync_update_product( $sync_options ) {
		global $wpdb;

		$query = 'SELECT  `product_id` ,  `moodle_post_id` , `post_title` ,  `post_content` 
			FROM  `' . $wpdb->prefix . 'eb_moodle_course_products` moodle_course,  `' . $wpdb->prefix . 'posts` posts
			WHERE moodle_course.`moodle_post_id` = posts.`ID`';

		$result = $wpdb->get_results( $query ); // @codingStandardsIgnoreLine
		$response = array();

		$_POST['eb_product_sync'] = 1;

		if ( ! empty( $result ) ) {
			$this->edwiser_bridge->logger()->add( 'product', 'Update all associated Products ...' ); // Add product updated log.
			$covered = array();
			foreach ( $result as $single_result ) {
				if ( in_array( intval( $single_result->product_id ), $covered ) ) { // @codingStandardsIgnoreLine
					continue;
				} else {
					$covered[] = intval( $single_result->product_id );
				}

				$course_content  = isset( $single_result->post_content ) ? $single_result->post_content : '';
				$course_id       = isset( $single_result->moodle_post_id ) ? $single_result->moodle_post_id : '';
				$title           = isset( $single_result->post_title ) ? $single_result->post_title : '';
				$product_options = get_post_meta( $single_result->product_id, 'product_options', true );
				$moodle_courses  = isset( $product_options['moodle_post_course_id'] ) ? $product_options['moodle_post_course_id'] : array();

				$product_args = array(
					'ID'           => $single_result->product_id,
					'post_content' => $course_content,
				);
				if ( 1 === count( $moodle_courses ) ) {
					$product_args['post_title'] = $single_result->post_title;
				}
				wp_update_post( $product_args );

				// Add categories to the previously created products.

				$category_args = array(
					'orderby' => 'name',
					'order'   => 'ASC',
					'fields'  => 'slugs',
				);

				$course_categories = wp_get_object_terms( $course_id, 'eb_course_cat', $category_args );

				$this->set_obj_terms( $course_categories, $single_result->product_id );

				do_action( 'bridge_woo_course_product_updated', $single_result->product_id, $course_id, $sync_options );
			}

			$response['respone_message'] = '<div class="alert alert-success">' . __( 'Product( s ) updated.', 'edwiser-bridge-pro' ) . '</div>';
		} else {
			$response['respone_message'] = '<div class="alert alert-error">' . __( 'Product for update not found.', 'edwiser-bridge-pro' ) . '</div>';
		}

		return $response;
	}

	/**
	 * This function updates existing Course categories with WooCommerce categories
	 *
	 * @since   1.0.3
	 * @access  public
	 */
	public function bridge_woo_sync_product_categories() {

		$term_args           = array(
			'hide_empty' => false,
			'orderby'    => 'id',
			'order'      => 'ASC',
			'fields'     => 'id=>parent',
		);
		$terms_relation_list = get_terms( 'eb_course_cat', $term_args );
		if ( ! empty( $terms_relation_list ) ) {
			asort( $terms_relation_list );
			$term_args = array( 'hide_empty' => false );

			$terms = get_terms( 'eb_course_cat', $term_args );

			$terms = $this->object2array( $terms );

			$term_id_list = function_exists( 'array_column' ) ? array_column( $terms, 'term_id' ) : $this->wdm_array_column( $terms, 'term_id' );

			$product_cat_relation = array();

			foreach ( $terms_relation_list as $key => $value ) {
				/*
					* Check term exists - IF yes update it, otherwise create new term
					*
					*/

				$search_key = array_search( $key, $term_id_list ); // @codingStandardsIgnoreLine

				$single_term = $terms[ $search_key ];

				if ( $value > 0 ) {
					// Term Has parent , check term exist with respect to this.

					if ( ! isset( $product_cat_relation[ $value ] ) ) {
						continue;
					}

					$term_exist_result = term_exists( $single_term['name'], 'product_cat', $product_cat_relation[ $value ] );

					if ( 0 == $term_exist_result || null == $term_exist_result ) { // @codingStandardsIgnoreLine
						$term_created = wp_insert_term( $single_term['name'], 'product_cat', array( 'parent' => $product_cat_relation[ $value ], 'description' => $single_term['description'], 'slug' => $single_term['slug'] ) ); // @codingStandardsIgnoreLine

						if ( ! is_wp_error( $term_created ) ) {
							$product_cat_relation[ $single_term['term_id'] ] = $term_created['term_id'];
						}
					} else {
						// Update Term.

						$term_updated = wp_update_term( $term_exist_result['term_id'], 'product_cat', array( 'description' => $single_term['description'], 'slug' => $single_term['slug'] ) ); // @codingStandardsIgnoreLine

						if ( ! is_wp_error( $term_updated ) ) {
							$product_cat_relation[ $single_term['term_id'] ] = $term_updated['term_id'];
						}
					}
				} else {
					// check if term exist.

					$term_exist_result = term_exists( $single_term['name'], 'product_cat' );

					if ( 0 == $term_exist_result || null == $term_exist_result ) { // @codingStandardsIgnoreLine
						// Insert Term.

						$term_created = wp_insert_term( $single_term['name'], 'product_cat', array( 'description' => $single_term['description'], 'slug' => $single_term['slug'] ) ); // @codingStandardsIgnoreLine

						if ( ! is_wp_error( $term_created ) ) {
							$product_cat_relation[ $single_term['term_id'] ] = $term_created['term_id'];
						}
					} else {
						// Update Term.

						$term_updated = wp_update_term( $term_exist_result['term_id'], 'product_cat', array( 'description' => $single_term['description'], 'slug' => $single_term['slug'] ) ); // @codingStandardsIgnoreLine

						if ( ! is_wp_error( $term_updated ) ) {
							$product_cat_relation[ $single_term['term_id'] ] = $term_updated['term_id'];
						}
					}
				}
			}//foreach ends

			$this->edwiser_bridge->logger()->add( 'product', 'Categories synchronized' ); // Add product created log.

			$response['respone_message'] = '<div class="alert alert-success">' . __( 'Categories synchronized.', 'edwiser-bridge-pro' ) . '</div>';
		} else {
			$response['respone_message'] = '<div class="alert alert-error">' . __( 'Categories not found, Please synchronize course categories before this.', 'edwiser-bridge-pro' ) . '</div>';
		}
		return $response;
	} //function ends - bridge_woo_sync_product_categories.

	/**
	 * This function converts object to array
	 *
	 * @param object $obj Object to be converted.
	 * @since 1.0.3
	 * @return $array Array - converted object
	 */
	private function object2array( $obj ) {
		if ( is_object( $obj ) || is_array( $obj ) ) {

			$obj = is_object( $obj ) ? get_object_vars( $obj ) : $obj;
			$obj = array_map( array( $this, 'object2array' ), $obj );
		}
		return $obj;
	}

	/**
	 * This is AJAX Handler for Product synchronization
	 *
	 * @param array $sync_options       synchronization option selected by User.
	 *
	 * @since   1.0.2
	 * @access  public
	 */
	public function bridge_woo_product_sync_handler( $sync_options ) {
		$response         = array();
		$category_respone = array();
		if ( isset( $sync_options['bridge_woo_synchronize_product_categories'] ) && 1 == $sync_options['bridge_woo_synchronize_product_categories'] ) { // @codingStandardsIgnoreLine
			$category_respone = $this->bridge_woo_sync_product_categories();
		}

		// Update Products.

		if ( isset( $sync_options['bridge_woo_synchronize_product_update'] ) && 1 == $sync_options['bridge_woo_synchronize_product_update'] ) { // @codingStandardsIgnoreLine
			$response = $this->bridge_woo_sync_update_product( $sync_options );
		}

		// Create Products.

		if ( isset( $sync_options['bridge_woo_synchronize_product_create'] ) && 1 == $sync_options['bridge_woo_synchronize_product_create'] ) { // @codingStandardsIgnoreLine
			if ( empty( $response ) ) {
				$response = $this->bridge_woo_sync_create_product( $sync_options );
			} else {
				$response = array_merge_recursive( $response, $this->bridge_woo_sync_create_product( $sync_options ) );
			}
		}
		if ( ! empty( $category_respone ) ) {
			$response = array_merge_recursive( $response, $category_respone );
		}
		return $response;
	}

	/**
	 * Provides support for PHP - array_column() function
	 *
	 * @since 1.0.1
	 * @param array $input      The multi-dimensional array (record set) from which to pull a column of values.
	 * @param mixed $column_key The column of values to return. This value may be the integer key of the column you wish to retrieve, or it may be the string key name for an associative array.
	 * @param mixed $index_key  The column to use as the index/keys for the returned array. This value may be the integer key of the column, or it may be the string key name.
	 */
	public function wdm_array_column( $input = null, $column_key = null, $index_key = null ) {
		if ( empty( $input ) ) {
			$input = null;
		}
		if ( empty( $column_key ) ) {
			$column_key = null;
		}
		if ( empty( $index_key ) ) {
			$index_key = null;
		}

		// Using func_get_args() in order to check for proper number of
		// parameters and trigger errors exactly as the built-in array_column()
		// does in PHP 5.5.
		$argc              = func_num_args();
		$params            = func_get_args();
		$params_input      = $params[0];
		$params_column_key = ( null !== $params[1] ) ? (string) $params[1] : null;
		$params_index_key  = null;
		if ( isset( $params[2] ) ) {
			if ( is_float( $params[2] ) || is_int( $params[2] ) ) {
				$params_index_key = (int) $params[2];
			} else {
				$params_index_key = (string) $params[2];
			}
		}
		$result_array = array();

		$result_array = $this->get_result_arr( $params_input, $params_index_key, $params_column_key );

		return $result_array;
	}

	/**
	 * This function is used to get the result array
	 *
	 * @since 1.0.1
	 * @param array  $params_input      The multi-dimensional array (record set) from which to pull a column of values.
	 * @param string $params_index_key  The column to use as the index/keys for the returned array. This value may be the integer key of the column, or it may be the string key name.
	 * @param string $params_column_key The column of values to return. This value may be the integer key of the column you wish to retrieve, or it may be the string key name for an associative array.
	 * @return array $result_array
	 */
	public function get_result_arr( $params_input, $params_index_key, $params_column_key ) {
		$result_array = array();

		foreach ( $params_input as $row ) {
			$key       = null;
			$value     = null;
			$key_set   = false;
			$value_set = false;
			if ( null !== $params_index_key && array_key_exists( $params_index_key, $row ) ) {
				$key_set = true;
				$key     = (string) $row[ $params_index_key ];
			}
			if ( null === $params_column_key ) {
				$value_set = true;
				$value     = $row;
			} elseif ( is_array( $row ) && array_key_exists( $params_column_key, $row ) ) {
				$value_set = true;
				$value     = $row[ $params_column_key ];
			}
			if ( $value_set ) {
				if ( $key_set ) {
					$result_array[ $key ] = $value;
				} else {
					$result_array[] = $value;
				}
			}
		}
		return $result_array;
	}
}
